==> aim-transport-0.9.12.tar.gz: aim-transport-0.9.12/src/messages.c <==
#include "aimtrans.h"

int at_parse_incoming_im(struct aim_session_t *ass, 
                         struct command_rx_struct *command, ...)
{
	int channel;
	va_list ap;

	va_start(ap, command);
	channel = va_arg(ap, int);

	/*
	* Channel 1: Standard Message
	*/
	if (channel == 1)
	{
    	struct aim_userinfo_s *userinfo;
	    char *msg = NULL;
        char *fullmsg;
    	u_int icbmflags = 0;
    	u_short flag1, flag2;
		jpacket jp;
		xmlnode par;
		xmlnode node;
		at_session s;
        ati ti;
		jid from;
  
	    userinfo = va_arg(ap, struct aim_userinfo_s *);
    	msg = va_arg(ap, char *);
	    icbmflags = va_arg(ap, u_int);
	    flag1 = va_arg(ap, u_int);
	    flag2 = va_arg(ap, u_int);
    	va_end(ap);

		s = (at_session)ass->aux_data;
        ti = s->ti;


		par = xmlnode_new_tag("message");
		xmlnode_put_attrib(par, "type", "chat");
		xmlnode_put_attrib(par, "to", jid_full(s->cur));
		xmlnode_put_attrib(par, "from", ti->i->id);
		jp = jpacket_new(par);
		node = xmlnode_insert_tag(jp->x, "body");
        msg = (char *)strip_html(msg, jp->p);
		msg = (char *)str_to_UTF8(jp->p, (unsigned char *)msg);
        fullmsg = pmalloco(xmlnode_pool(node), strlen(msg) + 30);
        if(icbmflags & AIM_IMFLAGS_AWAY)
        {
            strcat(fullmsg, "(AutoReply) ");
        }
        strcat(fullmsg, msg);
        if(s->away && (!(icbmflags & AIM_IMFLAGS_AWAY)))
        {
            aim_send_im(ass, command->conn, userinfo->sn, AIM_IMFLAGS_AWAY, s->status);
            strcat(fullmsg, " (Sent AutoReply)");
        }
		xmlnode_insert_cdata(node, fullmsg, strlen(fullmsg));


		jid_set(jp->from, at_normalize((char *)userinfo->sn), JID_USER);
                xmlnode_put_attrib(jp->x,"from",jid_full(jp->from));

		log_debug(ZONE, "[AIM] Sending: %s\n", xmlnode2str(jp->x));
        deliver(dpacket_new(jp->x), ti->i);

        pth_wait(NULL);
	}
	else if(channel == 2)
	{
		/* XXX decide how to handle groupchat requests */
		/* XXX Actualy parse and use this */
	}

  return 1;
}

int at_parse_misses(struct aim_session_t *sess, 
                    struct command_rx_struct *command, ...)
{
    xmlnode x;
    xmlnode body;
    at_session s;
    ati ti;

    static char *missedreasons[] = {
        "Unknown",
        "Message too large"};
    static int missedreasonslen = 2;

    va_list ap;
    unsigned short chan, nummissed, reason;
    struct aim_userinfo_s *userinfo;
    char msg[1024];

    s = (at_session)sess->aux_data;

	x = xmlnode_new_tag("message");
	xmlnode_put_attrib(x, "to", jid_full(s->cur));
	xmlnode_put_attrib(x, "from", jid_full(s->from));
	xmlnode_put_attrib(x, "type","error");
	body = xmlnode_insert_tag(x, "error");
  
    va_start(ap, command);
    chan = va_arg(ap, u_int);
    userinfo = va_arg(ap, struct aim_userinfo_s *);
    nummissed = va_arg(ap, u_int);
    reason = va_arg(ap, u_int);
    va_end(ap);

    memset(&msg, '\0', 1024);
    snprintf((char *)&msg, 1024, "Missed %d messages from %s (reason %d: %s)", 
             nummissed, userinfo->sn, reason, 
             (reason<missedreasonslen)?missedreasons[reason]:"unknown");
  
    xmlnode_insert_cdata(body, (char *)&msg, strlen((char *)&msg));

    deliver(dpacket_new(x), ti->i);
    return 1;
}

int at_parse_msgerr(struct aim_session_t *sess, struct command_rx_struct *command, ...)
{
    xmlnode x;
    xmlnode body;
    jpacket jp;
    at_session s;
    ati ti;
    char msg[1024];

    va_list ap;
    char *destsn;
    unsigned short reason;

    va_start(ap, command);
    reason = va_arg(ap, u_int);
    destsn = va_arg(ap, char *);
    va_end(ap);

    memset((char *)&msg, '\0', 1024);
    snprintf((char *)&msg, 1024, 
             "faimtest: message to %s bounced (reason 0x%04x: %s)\n", 
             destsn, reason, 
             (reason<msgerrreasonslen)?msgerrreasons[reason]:"unknown");

    s = (at_session)sess->aux_data;
    ti = s->ti;

	x = xmlnode_new_tag("message");
	xmlnode_put_attrib(x, "to", jid_full(s->cur));
	xmlnode_put_attrib(x, "from", jid_full(s->from));
	xmlnode_put_attrib(x, "type","error");
	body = xmlnode_insert_tag(x, "error");
    xmlnode_insert_cdata(body, (char *)&msg, strlen((char *)&msg));

    deliver(dpacket_new(x), ti->i);
  
    return 1;
}

==> aim-transport-0.9.15.tar.gz: aim-transport-0.9.15/src/messages.c <==
#include "aimtrans.h"

int at_parse_incoming_im(struct aim_session_t *ass, 
                         struct command_rx_struct *command, ...)
{
	int channel;
	va_list ap;

	va_start(ap, command);
	channel = va_arg(ap, int);

	/*
	* Channel 1: Standard Message
	*/
	if (channel == 1)
	{
    	struct aim_userinfo_s *userinfo;
	    char *msg = NULL;
        char *fullmsg;
    	u_int icbmflags = 0;
    	u_short flag1, flag2;
		jpacket jp;
		xmlnode par;
		xmlnode node;
		at_session s;
        ati ti;
		jid from;
  
	    userinfo = va_arg(ap, struct aim_userinfo_s *);
    	msg = va_arg(ap, char *);
	    icbmflags = va_arg(ap, u_int);
	    flag1 = va_arg(ap, u_int);
	    flag2 = va_arg(ap, u_int);
    	va_end(ap);

		s = (at_session)ass->aux_data;
        ti = s->ti;


		par = xmlnode_new_tag("message");
		xmlnode_put_attrib(par, "type", "chat");
		xmlnode_put_attrib(par, "to", jid_full(s->cur));
		xmlnode_put_attrib(par, "from", ti->i->id);
		jp = jpacket_new(par);
		node = xmlnode_insert_tag(jp->x, "body");
        msg = (char *)strip_html(msg, jp->p);
		msg = (char *)str_to_UTF8(jp->p, (unsigned char *)msg);
        fullmsg = pmalloco(xmlnode_pool(node), strlen(msg) + 30);
        if(icbmflags & AIM_IMFLAGS_AWAY)
        {
            strcat(fullmsg, "(AutoReply) ");
        }
        strcat(fullmsg, msg);
        if(s->away && (!(icbmflags & AIM_IMFLAGS_AWAY)))
        {
            aim_send_im(ass, command->conn, userinfo->sn, AIM_IMFLAGS_AWAY, s->status, strlen(s->status));
            strcat(fullmsg, " (Sent AutoReply)");
        }
		xmlnode_insert_cdata(node, fullmsg, strlen(fullmsg));


		jid_set(jp->from, at_normalize((char *)userinfo->sn), JID_USER);
                xmlnode_put_attrib(jp->x,"from",jid_full(jp->from));

		log_debug(ZONE, "[AIM] Sending: %s\n", xmlnode2str(jp->x));
        deliver(dpacket_new(jp->x), ti->i);

        pth_wait(NULL);
	}
	else if(channel == 2)
	{
		/* XXX decide how to handle groupchat requests */
		/* XXX Actualy parse and use this */
	}

  return 1;
}

int at_parse_misses(struct aim_session_t *sess, 
                    struct command_rx_struct *command, ...)
{
    xmlnode x;
    xmlnode body;
    at_session s;
    ati ti;

    static char *missedreasons[] = {
        "Unknown",
        "Message too large"};
    static int missedreasonslen = 2;

    va_list ap;
    unsigned short chan, nummissed, reason;
    struct aim_userinfo_s *userinfo;
    char msg[1024];

    s = (at_session)sess->aux_data;

	x = xmlnode_new_tag("message");
	xmlnode_put_attrib(x, "to", jid_full(s->cur));
	xmlnode_put_attrib(x, "from", jid_full(s->from));
	xmlnode_put_attrib(x, "type","error");
	body = xmlnode_insert_tag(x, "error");
  
    va_start(ap, command);
    chan = va_arg(ap, u_int);
    userinfo = va_arg(ap, struct aim_userinfo_s *);
    nummissed = va_arg(ap, u_int);
    reason = va_arg(ap, u_int);
    va_end(ap);

    memset(&msg, '\0', 1024);
    snprintf((char *)&msg, 1024, "Missed %d messages from %s (reason %d: %s)", 
             nummissed, userinfo->sn, reason, 
             (reason<missedreasonslen)?missedreasons[reason]:"unknown");
  
    xmlnode_insert_cdata(body, (char *)&msg, strlen((char *)&msg));

    deliver(dpacket_new(x), ti->i);
    return 1;
}

int at_parse_msgerr(struct aim_session_t *sess, struct command_rx_struct *command, ...)
{
    xmlnode x;
    xmlnode body;
    jpacket jp;
    at_session s;
    ati ti;
    char msg[1024];

    va_list ap;
    char *destsn;
    unsigned short reason;

    va_start(ap, command);
    reason = va_arg(ap, u_int);
    destsn = va_arg(ap, char *);
    va_end(ap);

    memset((char *)&msg, '\0', 1024);
    snprintf((char *)&msg, 1024, 
             "faimtest: message to %s bounced (reason 0x%04x: %s)\n", 
             destsn, reason, 
             (reason<msgerrreasonslen)?msgerrreasons[reason]:"unknown");

    s = (at_session)sess->aux_data;
    ti = s->ti;

	x = xmlnode_new_tag("message");
	xmlnode_put_attrib(x, "to", jid_full(s->cur));
	xmlnode_put_attrib(x, "from", jid_full(s->from));
	xmlnode_put_attrib(x, "type","error");
	body = xmlnode_insert_tag(x, "error");
    xmlnode_insert_cdata(body, (char *)&msg, strlen((char *)&msg));

    deliver(dpacket_new(x), ti->i);
  
    return 1;
}
